#include "MeshLoader.h"
#include "AneuMeshLoader.h"
#include "exceptions.h"

#include <iostream>
#include <string>
#include <fstream>
#include <algorithm>
#include <locale.h>

setlocale(0,"russian");

MeshLoader* File_Format_Checker(const std::string& file) {
    while (file.back() == ' ')                       
        file.pop_back();
    MeshLoader* mesh_pointer = nullptr;
	
    std::size_t found = file.rfind(".aneu");
    if (found != std::string::npos && file.length() - found == 5) {
        std::cout << "File-format-checker was passed." << std::endl;
        mesh_pointer = new AneuMeshLoader;
        return mesh_pointer;
    }

    found = file.rfind(".neu");
    if (found != std::string::npos && file.length() - found == 4) {
        std::cout << "File-format-checker was passed." << std::endl;
        mesh_pointer = new AneuMeshLoader;
        return mesh_pointer;
    }

    else {
        std::cout << "Warning! File-format-checker was not passed!" << std::endl;
        std::size_t dot = file.rfind('.');
        if (dot == std::string::npos)
            throw BadFileFormatException();
        else throw BadFileFormatException(file.substr(file.rfind('.'), file.size() - file.rfind('.')));
    };
}

void Mesh_Loader_Demo(MeshLoader* loader_, const std::string& filepath_) {
	if(loader_ != 0)
	{
	AneuMeshLoader* loader__;
	if (loader__ = dynamic_cast<AneuMeshLoader*>(loader_))
	{
	     loader_->LoadMesh(filepath_);
         loader_->Print_Data();
        std::vector<FiniteElement> elements_vector;
        std::vector<Node> nodes_vector;
        std::vector<std::vector<Node>> neighbors;
	    std::vector<BoundaryFiniteElement> surfaces_vector;

        std::cout << "Result of find_Elements_by_nodes(2, 4, 8) function:" << std::endl;                
        elements_vector = loader_->find_Elements_by_nodes(2, 4, 8);
        std::cout << elements_vector;
        elements_vector.clear();

        std::cout << "\nResult of find_Elements_by_edge(2, 8) function:\n";
        elements_vector = loader_->find_Elements_by_edge(2, 8);
        std::cout << elements_vector;
        elements_vector.clear();

        std::cout << "\nResult of find_Nodes_by_s_id(6) function:\n";
        nodes_vector = loader_->find_Nodes_by_s_id(6);
        std::cout << nodes_vector;
        nodes_vector.clear();

        std::cout << "\nResult of find_Elements_by_material(1) function:\n";
        elements_vector = loader_->find_Elements_by_material(1);
        std::cout << elements_vector;
        elements_vector.clear();

        std::cout << "\nResult of find_Belements_by_b_id(1) function:\n";
        surfaces_vector = loader_->find_Belements_by_b_id(1);
        std::cout << surfaces_vector;
        surfaces_vector.clear();

        std::cout << "\nResult of get_nodes_neighbors() function:\n";
        neighbors = loader_->get_nodes_neighbors();
        for (int i = 1; i < neighbors.size(); ++i)
            std::cout << "Neighbors for Node with ID = " << i << "\n" << neighbors[i];
        neighbors.clear();

        std::cout << "\nResult of Transform_Elements_to_Quadratic() function:\n";
        loader_->Transform_Elements_to_Quadratic();
        loader_->Print_Data();
	}
	}
}

struct fixed_tolower : std::unary_function<char, char> { 
    char operator()(char c) const {
        return tolower((unsigned char)c);
    }
};


int main() {
    std::string filepath = ".\\test.aneu";
	if (filepath.size() == 0)
        std::cout << "No arguments (File path)!!!" << std::endl;
    else {
		std::cout << "Set file path to \"" << filepath << "\"..." << std::endl;
        std::transform(filepath.begin(), filepath.end(), filepath.begin(), fixed_tolower());
    };

    MeshLoader* loader = File_Format_Checker(filepath);
    
    Mesh_Loader_Demo(loader, filepath);
    delete loader;
	system("pause");
    return 0;
}
